/*=============================================================================
                             CursesTyp.h
===============================================================================
  Define types that are missing from the Curses library, which other Curses
  libraries define.  This way, we'll have a set of types that we can use
  regardless of which Curses library we have.
=============================================================================*/

/* The C_xxx macros are defined by CursesDef.h, which was generated by
   the build system based on its analysis of the Curses library on this
   system.

   C_xxx defined means the Curses library header files define type 'xxx'.
*/

#ifndef C_TYPATTR_T
#define attr_t int
#endif

#if 0
/* Disabled this in Curses.pm 1.21.  Why would we want 'bool' defined? */
#ifndef C_TYPBOOL
#define bool int
#endif
#endif

#ifndef C_TYPCHTYPE
#define chtype int
#endif

#ifndef C_TYPMEVENT
#define MEVENT int
#endif

#ifndef C_TYPMMASK_T
#define mmask_t int
#endif

#ifndef C_TYPSCREEN
#define SCREEN int
#endif



/*
**
**  Copyright (c) 1994-2001  William Setzer
**
**  You may distribute under the terms of either the Artistic License
**  or the GNU General Public License, as specified in the README file.
*/

